<?php

namespace Drupal\engagebay\Service;

use Drupal;
use Exception;
use GuzzleHttp\Client;

class EngageBayAPI implements EngageBayAPIInterface
{
    private const CONTENT_TYPE_FORM = 'application/x-www-form-urlencoded';
    private const CONTENT_TYPE_JSON = 'application/json';

    private ?Client $client;

    private array $headers = [
        'accept' => self::CONTENT_TYPE_JSON,
        'content-type' => 'application/json'
    ];

    public function __construct(string $restApiKey = null)
    {
        $this->client = Drupal::httpClient();

        if ($restApiKey) {
            $this->headers['Authorization'] = $restApiKey;
        }
    }

    public function login(string $username, string $password): ?array
    {
        $this->headers['content-type'] = 'application/x-www-form-urlencoded';
        $this->headers['accept'] = '*/*';

        $body = ['email' => $username, 'password' => $password, 'source' => 'WORDPRESS',];

        return $this->post(self::DOMAIN_LOGIN, $body, false);
    }

    public function getLandingPages(): ?array
    {
        return $this->get(self::GET_LANDING_PAGES);
    }

    public function getForms(): ?array
    {
        return $this->get(self::GET_FORMS);
    }

    public function getLandingPage(string $landing_page_id): ?array
    {
        return $this->get(sprintf(self::GET_LANDING_PAGES, $landing_page_id));
    }

    public function post(string $uri, array $data): ?array
    {
        return $this->sendRequest('POST', $uri, $data);
    }

    public function get(string $uri): ?array
    {
        return $this->sendRequest('GET', $uri, []);
    }

    public function sendRequest(string $type, string $uri, array $data): ?array
    {
        if ($this->headers['content-type'] === self::CONTENT_TYPE_FORM) {
            $body_key = 'form_params';
        } else {
            $body_key = 'body';
        }

        try {
            $response = $this->client->request($type, $uri, [
                'headers' => $this->headers,
                'form_params' => $data
            ]);

            $response_body = $response->getBody();
            if ($response->getStatusCode() === 200) {
                return $this->isJson($response_body) ? json_decode($response_body, JSON_PRETTY_PRINT) : $response_body;
            }

        } catch (Exception $e) {
            watchdog_exception('engagebay', $e, $e->getMessage());
        }

        return null;
    }

    private function isJson(string $str): bool
    {
        json_decode($str);

        return json_last_error() === JSON_ERROR_NONE;
    }
}
